@@ -266,6 +266,9 @@ class _Activator(object):
         self.command = command
 
     def _yield_commands(self, cmds_dict):
+        for key, value in sorted(iteritems(cmds_dict.get('export_path', {}))):
+            yield self.export_var_tmpl % (key, value)
+
         for script in cmds_dict.get('deactivate_scripts', ()):
             yield self.run_script_tmpl % script
 
@@ -364,6 +367,7 @@ class _Activator(object):
         }
 
     def build_deactivate(self):
+        self._deactivate = True
         # query environment
         old_conda_prefix = self.environ.get('CONDA_PREFIX')
         old_conda_shlvl = int(self.environ.get('CONDA_SHLVL', '').strip() or 0)
@@ -390,13 +394,13 @@ class _Activator(object):
             # deactivated conda and anything at all in my env still references it (apart from the
             # shell script, we need something I suppose!)
             export_vars, unset_vars = self.get_export_unset_vars(odargs=OrderedDict((
-                ('path', new_path),
                 ('conda_prefix', None),
                 ('conda_shlvl', new_conda_shlvl),
                 ('conda_default_env', None),
                 ('conda_prompt_modifier', None))))
             conda_prompt_modifier = ''
             activate_scripts = ()
+            export_path = {'PATH': new_path, }
         else:
             assert old_conda_shlvl > 1
             new_prefix = self.environ.get('CONDA_PREFIX_%d' % new_conda_shlvl)
@@ -416,13 +420,12 @@ class _Activator(object):
                 )
 
             export_vars, unset_vars2 = self.get_export_unset_vars(odargs=OrderedDict((
-                ('path', new_path),
                 ('conda_prefix', new_prefix),
                 ('conda_shlvl', new_conda_shlvl),
                 ('conda_default_env', conda_default_env),
                 ('conda_prompt_modifier', conda_prompt_modifier))))
             unset_vars += unset_vars2
-
+            export_path = {'PATH': new_path, }
             activate_scripts = self._get_activate_scripts(new_prefix)
 
         if context.changeps1:
@@ -432,11 +435,13 @@ class _Activator(object):
             'unset_vars': unset_vars,
             'set_vars': set_vars,
             'export_vars': export_vars,
+            'export_path': export_path,
             'deactivate_scripts': deactivate_scripts,
             'activate_scripts': activate_scripts,
         }
 
     def build_reactivate(self):
+        self._reactivate = True
         conda_prefix = self.environ.get('CONDA_PREFIX')
         conda_shlvl = int(self.environ.get('CONDA_SHLVL', '').strip() or 0)
         if not conda_prefix or conda_shlvl < 1:
@@ -592,8 +597,44 @@ class _Activator(object):
 
     def _prompt_modifier(self, prefix, conda_default_env):
         if context.changeps1:
+            # Get current environment and prompt stack
+            env_stack = []
+            prompt_stack = []
+            old_shlvl = int(self.environ.get('CONDA_SHLVL', '0').rstrip())
+            for i in range(1, old_shlvl+1):
+                if i == old_shlvl:
+                    env_i = self._default_env(self.environ.get('CONDA_PREFIX', ''))
+                else:
+                    env_i = self._default_env(
+                            self.environ.get('CONDA_PREFIX_{}'.format(i), '').rstrip())
+                stacked_i = bool(self.environ.get('CONDA_STACKED_{}'.format(i), '').rstrip())
+                env_stack.append(env_i)
+                if not stacked_i:
+                    prompt_stack = prompt_stack[0:-1]
+                prompt_stack.append(env_i)
+
+            # Modify prompt stack according to pending operation
+            deactivate = getattr(self, '_deactivate', False)
+            reactivate = getattr(self, '_reactivate', False)
+            if deactivate:
+                prompt_stack = prompt_stack[0:-1]
+                env_stack = env_stack[0:-1]
+                stacked = bool(self.environ.get('CONDA_STACKED_{}'.format(old_shlvl), '').rstrip())
+                if not stacked and env_stack:
+                    prompt_stack.append(env_stack[-1])
+            elif reactivate:
+                pass
+            else:
+                stack = getattr(self, 'stack', False)
+                if not stack:
+                    prompt_stack = prompt_stack[0:-1]
+                prompt_stack.append(conda_default_env)
+
+            conda_stacked_env = ','.join(prompt_stack[::-1])
+
             return context.env_prompt.format(
                 default_env=conda_default_env,
+                stacked_env=conda_stacked_env,
                 prefix=prefix,
                 name=basename(prefix),
             )
@@ -797,7 +838,7 @@ class XonshActivator(_Activator):
         self.sep = '/'
         self.path_conversion = native_path_to_unix
         self.script_extension = '.xsh'
-        self.tempfile_extension = '.xsh'
+        self.tempfile_extension = None
         self.command_join = '\n'
 
         self.unset_var_tmpl = 'del $%s'
@@ -810,7 +851,7 @@ class XonshActivator(_Activator):
         super(XonshActivator, self).__init__(arguments)
 
     def _hook_preamble(self):
-        return 'CONDA_EXE = "%s"' % context.conda_exe
+        return '$CONDA_EXE = "%s"' % context.conda_exe
 
 
 class CmdExeActivator(_Activator):
