@@ -414,7 +414,7 @@ def make_install_plan(conda_prefix):
         plan.append({
             'function': install_conda_xsh.__name__,
             'kwargs': {
-                'target_path': join(site_packages_dir, 'xonsh', 'conda.xsh'),
+                'target_path': join(site_packages_dir, 'xontrib', 'conda.xsh'),
                 'conda_prefix': conda_prefix,
             },
         })
@@ -493,6 +493,32 @@ def make_initialize_plan(conda_prefix, shells, for_user, for_system, anaconda_pr
                 },
             })
 
+    if 'xonsh' in shells:
+        if for_user:
+            config_xonsh_path = expand(join('~', '.xonshrc'))
+            plan.append({
+                'function': init_xonsh_user.__name__,
+                'kwargs': {
+                    'target_path': config_xonsh_path,
+                    'conda_prefix': conda_prefix,
+                    'reverse': reverse,
+                },
+            })
+
+        if for_system:
+            if on_win:
+                config_xonsh_path = expand(join('%ALLUSERSPROFILE%', 'xonsh', 'xonshrc'))
+            else:
+                config_xonsh_path = '/etc/xonshrc'
+            plan.append({
+                'function': init_xonsh_user.__name__,
+                'kwargs': {
+                    'target_path': config_xonsh_path,
+                    'conda_prefix': conda_prefix,
+                    'reverse': reverse,
+                },
+            })
+
     if 'tcsh' in shells and for_user:
         tcshrc_path = expand(join('~', '.tcshrc'))
         plan.append({
@@ -634,7 +660,7 @@ def run_plan_elevated(plan):
 
     if any(step['result'] == Result.NEEDS_SUDO for step in plan):
         if on_win:
-            from ..common.os.windows import run_as_admin
+            from ..common._os.windows import run_as_admin
             temp_path = None
             try:
                 with Utf8NamedTemporaryFile('w+', suffix='.json', delete=False) as tf:
@@ -973,7 +999,7 @@ def _config_fish_content(conda_prefix):
     conda_initialize_content = dals("""
     # >>> conda initialize >>>
     # !! Contents within this block are managed by 'conda init' !!
-    eval (eval %(conda_exe)s "shell.fish" "hook" $argv)
+    eval %(conda_exe)s "shell.fish" "hook" $argv | source
     # <<< conda initialize <<<
     """) % {
         'conda_exe': conda_exe,
@@ -1066,6 +1092,91 @@ def init_fish_user(target_path, conda_prefix, reverse):
         return Result.NO_CHANGE
 
 
+def _config_xonsh_content(conda_prefix):
+    if on_win:
+        from ..activate import native_path_to_unix
+        conda_exe = native_path_to_unix(join(conda_prefix, 'Scripts', 'conda.exe'))
+    else:
+        conda_exe = join(conda_prefix, 'bin', 'conda')
+    conda_initialize_content = dals("""
+    # >>> conda initialize >>>
+    # !! Contents within this block are managed by 'conda init' !!
+    import sys as _sys
+    from types import ModuleType as _ModuleType
+    _mod = _ModuleType("xontrib.conda",
+                       "Autogenerated from $({conda_exe} shell.xonsh hook)")
+    __xonsh__.execer.exec($("{conda_exe}" "shell.xonsh" "hook"),
+                          glbs=_mod.__dict__,
+                          filename="$({conda_exe} shell.xonsh hook)")
+    _sys.modules["xontrib.conda"] = _mod
+    del _sys, _mod, _ModuleType
+    # <<< conda initialize <<<
+    """).format(conda_exe=conda_exe)
+    return conda_initialize_content
+
+
+def init_xonsh_user(target_path, conda_prefix, reverse):
+    # target_path: ~/.xonshrc
+    user_rc_path = target_path
+
+    try:
+        with open(user_rc_path) as fh:
+            rc_content = fh.read()
+    except FileNotFoundError:
+        rc_content = ''
+    except:
+        raise
+
+    rc_original_content = rc_content
+
+    conda_init_comment = "# commented out by conda initialize"
+    conda_initialize_content = _config_xonsh_content(conda_prefix)
+    if reverse:
+        # uncomment any lines that were commented by prior conda init run
+        rc_content = re.sub(
+            r"#\s(.*?)\s*{}".format(conda_init_comment),
+            r"\1",
+            rc_content,
+            flags=re.MULTILINE,
+        )
+
+        # remove any conda init sections added
+        rc_content = re.sub(
+            r"^\s*" + CONDA_INITIALIZE_RE_BLOCK,
+            "",
+            rc_content,
+            flags=re.DOTALL | re.MULTILINE
+        )
+    else:
+        replace_str = "__CONDA_REPLACE_ME_123__"
+        rc_content = re.sub(
+            CONDA_INITIALIZE_RE_BLOCK,
+            replace_str,
+            rc_content,
+            flags=re.MULTILINE,
+        )
+        # TODO: maybe remove all but last of replace_str, if there's more than one occurrence
+        rc_content = rc_content.replace(replace_str, conda_initialize_content)
+
+        if "# >>> conda initialize >>>" not in rc_content:
+            rc_content += '\n{0}\n'.format(conda_initialize_content)
+
+    if rc_content != rc_original_content:
+        if context.verbosity:
+            print('\n')
+            print(target_path)
+            print(make_diff(rc_original_content, rc_content))
+        if not context.dry_run:
+            # Make the directory if needed.
+            if not exists(dirname(user_rc_path)):
+                mkdir_p(dirname(user_rc_path))
+            with open(user_rc_path, 'w') as fh:
+                fh.write(rc_content)
+        return Result.MODIFIED
+    else:
+        return Result.NO_CHANGE
+
+
 def _bashrc_content(conda_prefix, shell):
     if on_win:
         from ..activate import native_path_to_unix
@@ -1081,27 +1192,44 @@ def _bashrc_content(conda_prefix, shell):
         }
     else:
         conda_exe = join(conda_prefix, 'bin', 'conda')
-        conda_initialize_content = dals("""
-        # >>> conda initialize >>>
-        # !! Contents within this block are managed by 'conda init' !!
-        __conda_setup="$('%(conda_exe)s' 'shell.%(shell)s' 'hook' 2> /dev/null)"
-        if [ $? -eq 0 ]; then
-            eval "$__conda_setup"
-        else
-            if [ -f "%(conda_prefix)s/etc/profile.d/conda.sh" ]; then
-                . "%(conda_prefix)s/etc/profile.d/conda.sh"
+        if shell in ("csh", "tcsh"):
+            conda_initialize_content = dals("""
+            # >>> conda initialize >>>
+            # !! Contents within this block are managed by 'conda init' !!
+            if ( -f "%(conda_prefix)s/etc/profile.d/conda.csh" ) then
+                source "%(conda_prefix)s/etc/profile.d/conda.csh"
             else
-                export PATH="%(conda_bin)s:$PATH"
+                setenv PATH="%(conda_bin)s:$PATH"
+            endif
+            # <<< conda initialize <<<
+            """) % {
+                'conda_exe': conda_exe,
+                'shell': shell,
+                'conda_bin': dirname(conda_exe),
+                'conda_prefix': conda_prefix,
+            }
+        else:
+            conda_initialize_content = dals("""
+            # >>> conda initialize >>>
+            # !! Contents within this block are managed by 'conda init' !!
+            __conda_setup="$('%(conda_exe)s' 'shell.%(shell)s' 'hook' 2> /dev/null)"
+            if [ $? -eq 0 ]; then
+                eval "$__conda_setup"
+            else
+                if [ -f "%(conda_prefix)s/etc/profile.d/conda.sh" ]; then
+                    . "%(conda_prefix)s/etc/profile.d/conda.sh"
+                else
+                    export PATH="%(conda_bin)s:$PATH"
+                fi
             fi
-        fi
-        unset __conda_setup
-        # <<< conda initialize <<<
-        """) % {
-            'conda_exe': conda_exe,
-            'shell': shell,
-            'conda_bin': dirname(conda_exe),
-            'conda_prefix': conda_prefix,
-        }
+            unset __conda_setup
+            # <<< conda initialize <<<
+            """) % {
+                'conda_exe': conda_exe,
+                'shell': shell,
+                'conda_bin': dirname(conda_exe),
+                'conda_prefix': conda_prefix,
+            }
     return conda_initialize_content
 
 
