@@ -25,8 +25,7 @@ from ..common.compat import iteritems, itervalues, odict, text_type
 from ..common.constants import NULL
 from ..common.io import Spinner, dashlist, time_recorder
 from ..common.path import get_major_minor_version, paths_equal
-from ..exceptions import (PackagesNotFoundError, SpecsConfigurationConflictError,
-                          ResolvePackageNotFound)
+from ..exceptions import PackagesNotFoundError, SpecsConfigurationConflictError, UnsatisfiableError
 from ..history import History
 from ..models.channel import Channel
 from ..models.enums import NoarchType
@@ -50,7 +49,7 @@ class Solver(object):
     """
 
     def __init__(self, prefix, channels, subdirs=(), specs_to_add=(), specs_to_remove=(),
-                 repodata_fn=REPODATA_FN):
+                 repodata_fn=REPODATA_FN, command=NULL):
         """
         Args:
             prefix (str):
@@ -72,6 +71,7 @@ class Solver(object):
         self.specs_to_add = frozenset(MatchSpec.merge(s for s in specs_to_add))
         self.specs_to_add_names = frozenset(_.name for _ in self.specs_to_add)
         self.specs_to_remove = frozenset(MatchSpec.merge(s for s in specs_to_remove))
+        self._command = command
 
         assert all(s in context.known_subdirs for s in self.subdirs)
         self._repodata_fn = repodata_fn
@@ -206,15 +206,23 @@ class Solver(object):
         ignore_pinned = context.ignore_pinned if ignore_pinned is NULL else ignore_pinned
         force_remove = context.force_remove if force_remove is NULL else force_remove
 
-        ssc = SolverStateContainer(
-            self.prefix, update_modifier, deps_modifier, prune, ignore_pinned, force_remove
-        )
-
         log.debug("solving prefix %s\n"
                   "  specs_to_remove: %s\n"
                   "  specs_to_add: %s\n"
                   "  prune: %s", self.prefix, self.specs_to_remove, self.specs_to_add, prune)
 
+        retrying = hasattr(self, 'ssc')
+
+        if not retrying:
+            ssc = SolverStateContainer(
+                self.prefix, update_modifier, deps_modifier, prune, ignore_pinned, force_remove
+            )
+            self.ssc = ssc
+        else:
+            ssc = self.ssc
+            ssc.update_modifier = update_modifier
+            ssc.deps_modifier = deps_modifier
+
         # force_remove is a special case where we return early
         if self.specs_to_remove and force_remove:
             if self.specs_to_add:
@@ -234,19 +242,33 @@ class Solver(object):
                 # Return early, with a solution that should just be PrefixData().iter_records()
                 return IndexedSet(PrefixGraph(ssc.solution_precs).graph)
 
-        with Spinner("Collecting package metadata (%s)" % self._repodata_fn,
-                     not context.verbosity and not context.quiet,
-                     context.json):
-            ssc = self._collect_all_metadata(ssc)
+        if not ssc.r:
+            with Spinner("Collecting package metadata (%s)" % self._repodata_fn,
+                         (not context.verbosity and not context.quiet and not retrying),
+                         context.json):
+                ssc = self._collect_all_metadata(ssc)
 
+        fail_message = ("failed\n" if self._repodata_fn == REPODATA_FN else "failed with %s, "
+                        "will retry with next repodata source.\n" % self._repodata_fn)
         with Spinner("Solving environment", not context.verbosity and not context.quiet,
-                     context.json):
+                     context.json, fail_message=fail_message):
             ssc = self._remove_specs(ssc)
             ssc = self._add_specs(ssc)
             solution_precs = copy.copy(ssc.solution_precs)
+
+            pre_packages = self.get_request_package_in_solution(ssc.solution_precs, ssc.specs_map)
             ssc = self._find_inconsistent_packages(ssc)
             # this will prune precs that are deps of precs that get removed due to conflicts
             ssc = self._run_sat(ssc)
+            post_packages = self.get_request_package_in_solution(ssc.solution_precs, ssc.specs_map)
+
+            if ssc.update_modifier == UpdateModifier.UPDATE_SPECS:
+                constrained = self.get_constrained_packages(
+                    pre_packages, post_packages, ssc.index.keys())
+                if len(constrained) > 0:
+                    for spec in constrained:
+                        self.determine_constricting_specs(spec, ssc.solution_precs)
+
             # if there were any conflicts, we need to add their orphaned deps back in
             if ssc.add_back_map:
                 orphan_precs = (set(solution_precs)
@@ -254,7 +276,6 @@ class Solver(object):
                                 - set(ssc.add_back_map))
                 ssc.solution_precs.extend([_ for _ in orphan_precs if _.name not in ssc.specs_map])
             ssc = self._post_sat_handling(ssc)
-            ssc = self._check_solution(ssc)
 
         time_recorder.log_totals()
 
@@ -266,12 +287,80 @@ class Solver(object):
 
         return ssc.solution_precs
 
+    def determine_constricting_specs(self, spec, solution_precs):
+        highest_version = [VersionOrder(sp.version) for sp in solution_precs
+                           if sp.name == spec.name][0]
+        constricting = []
+        for prec in solution_precs:
+            if any(j for j in prec.depends if spec.name in j):
+                for dep in prec.depends:
+                    m_dep = MatchSpec(dep)
+                    if m_dep.name == spec.name and \
+                            m_dep.version is not None and \
+                            (m_dep.version.exact_value or "<" in m_dep.version.spec):
+                        if "," in m_dep.version.spec:
+                            constricting.extend([
+                                (prec.name, MatchSpec("%s %s" % (m_dep.name, v)))
+                                for v in m_dep.version.tup if "<" in v.spec])
+                        else:
+                            constricting.append((prec.name, m_dep))
+
+        hard_constricting = [i for i in constricting if i[1].version.matcher_vo <= highest_version]
+        if len(hard_constricting) == 0:
+            return None
+
+        print("\n\nUpdating {spec} is constricted by \n".format(spec=spec.name))
+        for const in hard_constricting:
+            print("{package} -> requires {conflict_dep}".format(
+                package=const[0], conflict_dep=const[1]))
+        print("\nIf you are sure you want an update of your package either try "
+              "`conda update --all` or install a specific version of the "
+              "package you want using `conda install <pkg>=<version>`\n")
+        return hard_constricting
+
+    def get_request_package_in_solution(self, solution_precs, specs_map):
+        requested_packages = {}
+        for pkg in self.specs_to_add:
+            update_pkg_request = pkg.name
+
+            requested_packages[update_pkg_request] = [
+                (i.name, str(i.version)) for i in solution_precs
+                if i.name == update_pkg_request and i.version is not None
+            ]
+            requested_packages[update_pkg_request].extend(
+                [(v.name, str(v.version)) for k, v in specs_map.items()
+                 if k == update_pkg_request and v.version is not None])
+
+        return requested_packages
+
+    def get_constrained_packages(self, pre_packages, post_packages, index_keys):
+        update_constrained = set()
+
+        def empty_package_list(pkg):
+            for k, v in pkg.items():
+                if len(v) == 0:
+                    return True
+            return False
+
+        if empty_package_list(pre_packages) or empty_package_list(post_packages):
+            return update_constrained
+
+        for pkg in self.specs_to_add:
+            current_version = max(i[1] for i in pre_packages[pkg.name])
+            if current_version == max(i.version for i in index_keys if i.name == pkg.name):
+                continue
+            else:
+                if post_packages == pre_packages:
+                    update_constrained = update_constrained | set([pkg])
+        return update_constrained
+
     @time_recorder(module_name=__name__)
     def _collect_all_metadata(self, ssc):
         # add in historically-requested specs
         ssc.specs_map.update(ssc.specs_from_history_map)
 
-        for pkg_name in ('anaconda', 'conda', 'conda-build'):
+        for pkg_name in ('anaconda', 'conda', 'conda-build',
+                         'console_shortcut', 'powershell_shortcut'):
             if pkg_name not in ssc.specs_map and ssc.prefix_data.get(pkg_name, None):
                 ssc.specs_map[pkg_name] = MatchSpec(pkg_name)
 
@@ -388,70 +477,42 @@ class Solver(object):
                                        if prec not in inconsistent_precs)
         return ssc
 
-    def _get_package_pool(self, ssc, specs):
-        specs = frozenset(specs)
-        if specs in self._pool_cache:
-            pool = self._pool_cache[specs]
-        else:
-            pool = ssc.r.get_reduced_index(specs)
-            grouped_pool = groupby(lambda x: x.name, pool)
-            pool = {k: set(v) for k, v in iteritems(grouped_pool)}
-            self._pool_cache[specs] = pool
-        return pool
-
     def _package_has_updates(self, ssc, spec, installed_pool):
         installed_prec = installed_pool.get(spec.name)
-        has_update = installed_prec and any(_.version > installed_prec[0].version
-                                            for _ in ssc.r.groups.get(spec.name, []))
-        return (MatchSpec(spec.name, version=">" + str(installed_prec[0].version))
-                if has_update else None)
+        has_update = False
+
+        if installed_prec:
+            installed_prec = installed_prec[0]
+            for prec in ssc.r.groups.get(spec.name, []):
+                if prec.version > installed_prec.version:
+                    has_update = True
+                    break
+                elif (prec.version == installed_prec.version and
+                      prec.build_number > installed_prec.build_number):
+                    has_update = True
+                    break
+        # let conda determine the latest version by just adding a name spec
+        return MatchSpec(spec.name) if has_update else None
 
     def _should_freeze(self, ssc, target_prec, conflict_specs, explicit_pool, installed_pool):
         # never, ever freeze anything if we have no history.
         if not ssc.specs_from_history_map:
             return False
-
-        pkg_name = target_prec.name
-        # if we are FREEZE_INSTALLED (first pass for install)
-        freeze = ssc.update_modifier == UpdateModifier.FREEZE_INSTALLED
-
-        if not freeze:
-            # we are UPDATE_SPECS (solve_for_diff or update specs),
-            #       AND the spec is not an explicit spec
-            update_added = (ssc.update_modifier == UpdateModifier.UPDATE_SPECS and
-                            pkg_name not in self.specs_to_add_names)
-            # more expensive, but better check.  If anything in the package pool for this spec
-            #    overlaps with the explicit package pool (by name), but they don't share any
-            #    actual records, then this target_prec conflicts and should not be pinned
-            if update_added:
-                spec_package_pool = self._get_package_pool(ssc, (target_prec.to_match_spec(), ))
-                for spec in self.specs_to_add_names:
-                    new_explicit_pool = explicit_pool
-                    ms = MatchSpec(spec)
-                    updated_spec = self._package_has_updates(ssc, ms, installed_pool)
-                    if updated_spec:
-                        try:
-                            new_explicit_pool = self._get_package_pool(ssc, (updated_spec, ))
-                        except ResolvePackageNotFound:
-                            update_added = False
-                            break
-
-                    if ms.name in spec_package_pool:
-                        update_added &= bool(spec_package_pool[ms.name] &
-                                             new_explicit_pool[ms.name])
-                    if not update_added:
-                        break
+        # never freeze if not in FREEZE_INSTALLED mode
+        if ssc.update_modifier != UpdateModifier.FREEZE_INSTALLED:
+            return False
 
         # if all package specs have overlapping package choices (satisfiable in at least one way)
-        no_conflict = ((freeze or update_added) and
-                       pkg_name not in conflict_specs and
+        pkg_name = target_prec.name
+        no_conflict = (pkg_name not in conflict_specs and
                        (pkg_name not in explicit_pool or
                         target_prec in explicit_pool[pkg_name]) and
                        self._compare_pools(ssc, explicit_pool, target_prec.to_match_spec()))
+
         return no_conflict
 
     def _compare_pools(self, ssc, explicit_pool, ms):
-        other_pool = self._get_package_pool(ssc, (ms, ))
+        other_pool = ssc.r._get_package_pool((ms, ))
         match = True
         for k in set(other_pool.keys()) & set(explicit_pool.keys()):
             if not bool(other_pool[k] & explicit_pool[k]):
@@ -474,7 +535,7 @@ class Solver(object):
 
         # the only things we should consider freezing are things that don't conflict with the new
         #    specs being added.
-        explicit_pool = self._get_package_pool(ssc, self.specs_to_add)
+        explicit_pool = ssc.r._get_package_pool(self.specs_to_add)
 
         conflict_specs = ssc.r.get_conflicting_specs(list(concatv(
             self.specs_to_add, (_.to_match_spec() for _ in ssc.prefix_data.iter_records())))) or []
@@ -503,7 +564,9 @@ class Solver(object):
                                          installed_pool):
                     ssc.specs_map[pkg_name] = target_prec.to_match_spec()
                 elif pkg_name in ssc.specs_from_history_map:
-                    ssc.specs_map[pkg_name] = ssc.specs_from_history_map[pkg_name]
+                    ssc.specs_map[pkg_name] = MatchSpec(
+                        ssc.specs_from_history_map[pkg_name],
+                        target=target_prec.dist_str())
                 else:
                     ssc.specs_map[pkg_name] = MatchSpec(pkg_name, target=target_prec.dist_str())
 
@@ -512,7 +575,7 @@ class Solver(object):
             if s.name in explicit_pool:
                 if s.name not in self.specs_to_add_names and not ssc.ignore_pinned:
                     ssc.specs_map[s.name] = MatchSpec(s, optional=False)
-                elif explicit_pool[s.name] & self._get_package_pool(ssc, [s])[s.name]:
+                elif explicit_pool[s.name] & ssc.r._get_package_pool([s]).get(s.name, set()):
                     ssc.specs_map[s.name] = MatchSpec(s, optional=False)
                     pin_overrides.add(s.name)
                 else:
@@ -564,22 +627,49 @@ class Solver(object):
                                       for prec in ssc.prefix_data.iter_records()
                                       )
 
+        elif ssc.update_modifier == UpdateModifier.UPDATE_SPECS:
+            for spec in self.specs_to_add:
+                spec = MatchSpec(spec)
+                if (spec.name in pin_overrides and not ssc.ignore_pinned
+                        or spec.name in ssc.specs_from_history_map):
+                    # skip this spec, because it is constrained by pins
+                    continue
+                # the index is sorted, so the first record here gives us what we want.
+                latest_pkg = ssc.r.find_matches(spec)[0]
+                for ms in list(ssc.specs_map.values()):
+                    ms = MatchSpec(ms)
+                    spec_pool = ssc.r._get_package_pool([ms])
+                    if spec_pool.get(spec.name) and latest_pkg not in spec_pool[spec.name]:
+                        # neuter the spec due to a conflict
+                        ssc.specs_map[ms.name] = MatchSpec(ms.name)
+
         # As a business rule, we never want to update python beyond the current minor version,
         # unless that's requested explicitly by the user (which we actively discourage).
-        if (any(_.name == 'python' for _ in ssc.solution_precs)
-                and not any(s.name == 'python' for s in self.specs_to_add)):
-
-            # will our prefix record conflict with any explict spec?  If so, don't add
-            #     anything here - let python float when it hasn't been explicitly specified
+        py_in_prefix = any(_.name == 'python' for _ in ssc.solution_precs)
+        py_requested_explicitly = any(s.name == 'python' for s in self.specs_to_add)
+        if py_in_prefix and not py_requested_explicitly:
             python_prefix_rec = ssc.prefix_data.get('python')
-            if ('python' not in conflict_specs and
-                    ssc.update_modifier == UpdateModifier.FREEZE_INSTALLED):
+            freeze_installed = ssc.update_modifier == UpdateModifier.FREEZE_INSTALLED
+            if 'python' not in conflict_specs and freeze_installed:
                 ssc.specs_map['python'] = python_prefix_rec.to_match_spec()
             else:
+                # will our prefix record conflict with any explict spec?  If so, don't add
+                #     anything here - let python float when it hasn't been explicitly specified
                 python_spec = ssc.specs_map.get('python', MatchSpec('python'))
                 if not python_spec.get('version'):
                     pinned_version = get_major_minor_version(python_prefix_rec.version) + '.*'
-                    ssc.specs_map['python'] = MatchSpec(python_spec, version=pinned_version)
+                    python_spec = MatchSpec(python_spec, version=pinned_version)
+
+                spec_set = (python_spec, ) + tuple(self.specs_to_add)
+                if ssc.r.get_conflicting_specs(spec_set):
+                    if self._command != 'install' or (
+                            self._repodata_fn == REPODATA_FN and
+                            ssc.update_modifier != UpdateModifier.FREEZE_INSTALLED):
+                        # raises a hopefully helpful error message
+                        ssc.r.find_conflicts(spec_set)
+                    else:
+                        raise UnsatisfiableError({})
+                ssc.specs_map['python'] = python_spec
 
         # For the aggressive_update_packages configuration parameter, we strip any target
         # that's been set.
@@ -597,13 +687,14 @@ class Solver(object):
         if 'conda' in ssc.specs_map and paths_equal(self.prefix, context.conda_prefix):
             conda_prefix_rec = ssc.prefix_data.get('conda')
             if conda_prefix_rec:
+                version_req = ">=%s" % conda_prefix_rec.version
+                conda_requested_explicitly = any(s.name == 'conda' for s in self.specs_to_add)
                 conda_spec = ssc.specs_map['conda']
-
                 conda_in_specs_to_add_version = ssc.specs_map.get('conda', {}).get('version')
                 if not conda_in_specs_to_add_version:
-                    conda_spec = MatchSpec(conda_spec, version=">=%s" % conda_prefix_rec.version)
-                if context.auto_update_conda:
-                    conda_spec = MatchSpec('conda', target=None)
+                    conda_spec = MatchSpec(conda_spec, version=version_req)
+                if context.auto_update_conda and not conda_requested_explicitly:
+                    conda_spec = MatchSpec('conda', version=version_req, target=None)
                 ssc.specs_map['conda'] = conda_spec
 
         return ssc
@@ -616,6 +707,10 @@ class Solver(object):
             # pinned specs removed here - added to specs_map in _add_specs instead
         ))
 
+        absent_specs = [s for s in ssc.specs_map.values() if not ssc.r.find_matches(s)]
+        if absent_specs:
+            raise PackagesNotFoundError(absent_specs)
+
         # We've previously checked `solution` for consistency (which at that point was the
         # pre-solve state of the environment). Now we check our compiled set of
         # `final_environment_specs` for the possibility of a solution.  If there are conflicts,
@@ -681,7 +776,8 @@ class Solver(object):
                       dashlist(sorted(text_type(s) for s in final_environment_specs)))
         ssc.solution_precs = ssc.r.solve(tuple(final_environment_specs),
                                          specs_to_add=self.specs_to_add,
-                                         history_specs=ssc.specs_from_history_map)
+                                         history_specs=ssc.specs_from_history_map,
+                                         repodata_fn=self._repodata_fn)
 
         # add back inconsistent packages to solution
         if ssc.add_back_map:
@@ -877,6 +973,7 @@ class Solver(object):
             reduced_index = get_reduced_index(self.prefix, self.channels,
                                               self.subdirs, prepared_specs, self._repodata_fn)
             _supplement_index_with_system(reduced_index)
+
             self._prepared_specs = prepared_specs
             self._index = reduced_index
             self._r = Resolve(reduced_index, channels=self.channels)
@@ -884,29 +981,6 @@ class Solver(object):
         self._prepared = True
         return self._index, self._r
 
-    def _check_solution(self, ssc):
-        # Ensure that solution is consistent with pinned specs.
-        for spec in ssc.pinned_specs:
-            if spec.name in self.specs_to_add_names:
-                continue
-            spec = MatchSpec(spec, optional=False)
-            if not any(spec.match(d) for d in ssc.solution_precs):
-                # if the spec doesn't match outright, make sure there's no package by that
-                # name in the solution
-                assert not any(d.name == spec.name for d in ssc.solution_precs)
-
-                # Let this be handled as part of txn.verify()
-                # # Ensure conda or its dependencies aren't being uninstalled in conda's
-                # # own environment.
-                # if paths_equal(self.prefix, context.conda_prefix) and not context.force:
-                #     conda_spec = MatchSpec("conda")
-                #     conda_dist = next((conda_spec.match(d) for d in solution), None)
-                #     assert conda_dist
-                #     conda_deps_specs = self._r.ms_depends(conda_dist)
-                #     for spec in conda_deps_specs:
-                #         assert any(spec.match(d) for d in solution)
-        return ssc
-
 
 class SolverStateContainer(object):
     # A mutable container with defined attributes to help keep method signatures clean
