@@ -9,18 +9,20 @@ from logging import DEBUG, getLogger
 
 from ._vendor.auxlib.collection import frozendict
 from ._vendor.auxlib.decorators import memoize, memoizemethod
-from ._vendor.toolz import concat, concatv, groupby
-from .base.constants import ChannelPriority, MAX_CHANNEL_PRIORITY, SatSolverChoice
+from ._vendor.toolz import concat, groupby
+from .base.constants import ChannelPriority, MAX_CHANNEL_PRIORITY, SatSolverChoice, REPODATA_FN
 from .base.context import context
 from .common.compat import iteritems, iterkeys, itervalues, odict, on_win, text_type
 from .common.io import time_recorder
-from .common.logic import Clauses, CryptoMiniSatSolver, PycoSatSolver, PySatSolver
+from .common.logic import (Clauses, CryptoMiniSatSolver, PycoSatSolver, PySatSolver,
+                           minimal_unsatisfiable_subset)
 from .common.toposort import toposort
 from .exceptions import (CondaDependencyError, InvalidSpec, ResolvePackageNotFound,
                          UnsatisfiableError)
 from .models.channel import Channel, MultiChannel
-from .models.enums import NoarchType
+from .models.enums import NoarchType, PackageType
 from .models.match_spec import MatchSpec
+from .models.prefix_graph import GeneralGraph
 from .models.records import PackageRecord
 from .models.version import VersionOrder
 
@@ -109,8 +111,12 @@ class Resolve(object):
         self._cached_find_matches = {}  # Dict[MatchSpec, Set[PackageRecord]]
         self.ms_depends_ = {}  # Dict[PackageRecord, List[MatchSpec]]
         self._reduced_index_cache = {}
+        self._pool_cache = {}
         self._strict_channel_cache = {}
 
+        self._system_precs = {_ for _ in index if (
+            hasattr(_, 'package_type') and _.package_type == PackageType.VIRTUAL_SYSTEM)}
+
         # sorting these in reverse order is effectively prioritizing
         # contstraint behavior from newer packages. It is applying broadening
         # reduction based on the latest packages, which may reduce the space
@@ -214,19 +220,15 @@ class Resolve(object):
 
         A dependency chain is a tuple of MatchSpec objects, starting with
         the requested spec, proceeding down the dependency tree, ending at
-        a specification that cannot be satisfied. Uses self.valid_ as a
-        filter, both to prevent chains and to allow other routines to
-        prune the list of valid packages with additional criteria.
+        a specification that cannot be satisfied.
 
         Args:
             spec: a package key or MatchSpec
             filter: a dictionary of (prec, valid) pairs to be used when
                 testing for package validity.
-            optional: if True (default), do not enforce optional specifications
-                when considering validity. If False, enforce them.
 
         Returns:
-            A generator of tuples, empty if the MatchSpec is valid.
+            A tuple of tuples, empty if the MatchSpec is valid.
         """
         def chains_(spec, names):
             if spec.name in names:
@@ -236,31 +238,20 @@ class Resolve(object):
                 return
             precs = self.find_matches(spec)
             found = False
-            for prec in precs:
-                for m2 in self.ms_depends(prec):
-                    for x in chains_(m2, names):
-                        found = True
-                        yield (spec,) + x
-            if not found:
-                yield (spec,)
-        return chains_(spec, set())
 
-    def invalid_chains2(self, spec, filter_out, optional=True):
-        def chains_(spec, names):
-            if spec.name in names:
-                return
-            names.add(spec.name)
-            if self.valid2(spec, filter_out, optional):
-                return
-            precs = self.find_matches(spec)
-            found = False
+            conflict_deps = set()
             for prec in precs:
                 for m2 in self.ms_depends(prec):
                     for x in chains_(m2, names):
                         found = True
                         yield (spec,) + x
+                    else:
+                        conflict_deps.add(m2)
             if not found:
-                yield (spec,)
+                conflict_groups = groupby(lambda x: x.name, conflict_deps)
+                for group in conflict_groups.values():
+                    yield (spec,) + MatchSpec.union(group)
+
         return chains_(spec, set())
 
     def verify_specs(self, specs):
@@ -283,9 +274,8 @@ class Resolve(object):
                 feature_names.update(_feature_names)
             else:
                 non_tf_specs.append(ms)
-        filter = self.default_filter(feature_names)
-        for ms in non_tf_specs:
-            bad_deps.extend(self.invalid_chains(ms, filter.copy()))
+        bad_deps.extend((spec, ) for spec in non_tf_specs if (not spec.optional and
+                                                              not self.find_matches(spec)))
         if bad_deps:
             raise ResolvePackageNotFound(bad_deps)
         return tuple(non_tf_specs), feature_names
@@ -293,31 +283,46 @@ class Resolve(object):
     def _classify_bad_deps(self, bad_deps, specs_to_add, history_specs, strict_channel_priority):
         classes = {'python': set(),
                    'request_conflict_with_history': set(),
-                   'direct': set()}
+                   'direct': set(),
+                   'cuda': set(), }
         specs_to_add = set(MatchSpec(_) for _ in specs_to_add or [])
         history_specs = set(MatchSpec(_) for _ in history_specs or [])
 
         for chain in bad_deps:
             # sometimes chains come in as strings
-            chain = [MatchSpec(_) for _ in chain]
             if chain[-1].name == 'python' and len(chain) > 1 and \
+                    not any(_.name == 'python' for _ in specs_to_add) and \
                     any(_[0] for _ in bad_deps if _[0].name == 'python'):
                 python_first_specs = [_[0] for _ in bad_deps if _[0].name == 'python']
                 if python_first_specs:
                     python_spec = python_first_specs[0]
                     if not (set(self.find_matches(python_spec)) &
                             set(self.find_matches(chain[-1]))):
-                        classes['python'].add((tuple(chain), python_spec))
+                        classes['python'].add((tuple([chain[0], chain[-1]]),
+                                               str(MatchSpec(python_spec, target=None))))
+            elif chain[-1].name == '__cuda':
+                cuda_version = [_ for _ in self._system_precs if _.name == '__cuda']
+                cuda_version = cuda_version[0].version if cuda_version else "not available"
+                classes['cuda'].add((tuple(chain), cuda_version))
             elif chain[0] in specs_to_add:
                 match = False
                 for spec in history_specs:
                     if spec.name == chain[-1].name:
-                        classes['request_conflict_with_history'].add((tuple(chain), spec))
+                        classes['request_conflict_with_history'].add((
+                            tuple(chain), str(MatchSpec(spec, target=None))))
                         match = True
+
                 if not match:
-                    classes['direct'].add((tuple(chain), chain[0]))
+                    classes['direct'].add((tuple(chain), str(MatchSpec(chain[0], target=None))))
             else:
-                classes['direct'].add((tuple(chain), chain[0]))
+                if len(chain) > 1 or not any(len(c) > 1 and c[0] == chain[0] for c in bad_deps):
+                    classes['direct'].add((tuple(chain),
+                                           str(MatchSpec(chain[0], target=None))))
+
+        if classes['python']:
+            # filter out plain single-entry python conflicts.  The python section explains these.
+            classes['direct'] = [_ for _ in classes['direct']
+                                 if _[1].startswith('python ') or len(_[0]) > 1]
         return classes
 
     def find_matches_with_strict(self, ms, strict_channel_priority):
@@ -328,6 +333,11 @@ class Resolve(object):
         return tuple(f for f in matches if f.channel.name == sole_source_channel_name)
 
     def find_conflicts(self, specs, specs_to_add=None, history_specs=None):
+        bad_deps = self.build_conflict_map(specs, specs_to_add, history_specs)
+        strict_channel_priority = context.channel_priority == ChannelPriority.STRICT
+        raise UnsatisfiableError(bad_deps, strict=strict_channel_priority)
+
+    def build_conflict_map(self, specs, specs_to_add=None, history_specs=None):
         """Perform a deeper analysis on conflicting specifications, by attempting
         to find the common dependencies that might be the cause of conflicts.
 
@@ -357,96 +367,68 @@ class Resolve(object):
         """
         # if only a single package matches the spec use the packages depends
         # rather than the spec itself
+        strict_channel_priority = context.channel_priority == ChannelPriority.STRICT
+
+        specs = set(specs) | (specs_to_add or set())
         if len(specs) == 1:
             matches = self.find_matches(next(iter(specs)))
             if len(matches) == 1:
-                specs = self.ms_depends(matches[0])
+                specs = set(self.ms_depends(matches[0]))
+        specs.update({_.to_match_spec() for _ in self._system_precs})
 
-        strict_channel_priority = context.channel_priority == ChannelPriority.STRICT
-
-        sdeps = {}
         # For each spec, assemble a dictionary of dependencies, with package
         # name as key, and all of the matching packages as values.
-        for top_level_spec in specs:
-            # find all packages matching a top level specification
-            top_level_pkgs = self.find_matches_with_strict(top_level_spec, strict_channel_priority)
-            top_level_sdeps = {top_level_spec.name: set(top_level_pkgs)}
-
-            # find all depends specs for in top level packages
-            # find the depends names for each top level packages
-            second_level_specs = set()
-            top_level_pkg_dep_names = []
-            for pkg in top_level_pkgs:
-                pkg_deps = self.ms_depends(pkg)
-                second_level_specs.update(pkg_deps)
-                top_level_pkg_dep_names.append([d.name for d in pkg_deps])
-
-            # find all second level packages and their specs
-            slist = []
-            for ms in second_level_specs:
-                deps = top_level_sdeps.setdefault(ms.name, set())
-                for fkey in self.find_matches_with_strict(ms, strict_channel_priority):
-                    deps.add(fkey)
-                    slist.extend(
-                        ms2 for ms2 in self.ms_depends(fkey) if ms2.name != top_level_spec.name)
-
-            # dependency names which appear in all top level packages
-            # have been fully considered and not additions should be make to
-            # the package list for that name
-            locked_names = [top_level_spec.name]
-            if top_level_pkg_dep_names:
-                for name in top_level_pkg_dep_names[0]:
-                    if all(name in names for names in top_level_pkg_dep_names):
-                        locked_names.append(name)
-
-            # build out the rest of the dependency tree
-            while slist:
-                ms2 = slist.pop()
-                if ms2.name in locked_names:
-                    continue
-                deps = top_level_sdeps.setdefault(ms2.name, set())
-                for fkey in self.find_matches_with_strict(ms2, strict_channel_priority):
-                    if fkey not in deps:
-                        deps.add(fkey)
-                        slist.extend(ms3 for ms3 in self.ms_depends(fkey)
-                                     if ms3.name != top_level_spec.name)
-            sdeps[top_level_spec] = top_level_sdeps
+        sdeps = {k: self._get_package_pool((k, )) for k in specs}
 
         # find deps with zero intersection between specs which include that dep
         bad_deps = []
-        deps = set()
-        for sdep in sdeps.values():
-            deps.update(sdep.keys())
+        dep_collections = tuple(set(sdep.keys()) for sdep in sdeps.values())
+        deps = set.union(*dep_collections) if dep_collections else []
+        # for each possible package being considered, look at how pools interact
         for dep in deps:
-            sdeps_with_dep = {k: v.get(dep) for k, v in sdeps.items() if dep in v.keys()}
+            sdeps_with_dep = {}
+            for k, v in sdeps.items():
+                if dep in v:
+                    sdeps_with_dep[k] = v
             if len(sdeps_with_dep) <= 1:
                 continue
-            # if the two pools overlap, we're good.  Next dep.
-            if bool(set.intersection(*sdeps_with_dep.values())):
+            # if all of the pools overlap, we're good.  Next dep.
+            if bool(set.intersection(*[v[dep] for v in sdeps_with_dep.values()])):
                 continue
-            filter = {}
-            for fkeys in sdeps_with_dep.values():
-                for fkey in fkeys:
-                    filter[fkey] = False
-            for spec in sdeps_with_dep.keys():
-                ndeps = set(self.invalid_chains(spec, filter, False))
-                ndeps = [nd for nd in ndeps if nd[-1].name == dep]
-                bad_deps.extend(ndeps)
-        if not bad_deps:
-            for spec in specs:
-                filter = {}
-                for name, valid_pkgs in sdeps[spec].items():
-                    if name == spec.name:
-                        continue
-                    for fkey in self.find_matches(MatchSpec(name)):
-                        filter[fkey] = fkey in valid_pkgs
-                bad_deps.extend(self.invalid_chains(spec, filter, False))
+            # start out filtering nothing.  invalid_chains will tweak this dict to filter more
+            #    as it goes
+            records = set.union(*tuple(rec for records in sdeps_with_dep.values()
+                                       for rec in records.values()))
+            # determine the invalid chains for each specific spec.  Each of these chains
+            #    should start with `spec` and end with the first encountered conflict.  A
+            #    conflict is something that is either not available at all, or is present in
+            #    more than one pool, but those pools do not all overlap.
+
+            records_for_graph = groupby(lambda r: r.name,
+                                        (r for r in records if isinstance(r, PackageRecord)))
+            # seven is a completely arbitrary number here.  It is meant to gather more than just
+            #    one record, to explore the space of dependencies a bit.  Doing all of them
+            #    can be an enormous problem, though.  This is hopefully a good compromise.
+            g = GeneralGraph([_ for v in records_for_graph.values() for _ in v[:7]])
+            spec_order = sorted(sdeps_with_dep.keys(),
+                                key=lambda x: list(g.graph_by_name.keys()).index(x.name))
+            for spec in spec_order:
+                # the DFS approach works well when things are actually in the graph
+                bad_deps.extend(g.depth_first_search_by_name(spec, dep, sdeps[spec]))
+
         if not bad_deps:
             # no conflicting nor missing packages found, return the bad specs
-            bad_deps = [(ms, ) for ms in specs]
+            bad_deps = []
+
+            for spec in specs:
+                precs = self.find_matches(spec)
+                deps = set.union(*[set(self.ms_depends_.get(prec)) for prec in precs])
+                deps = groupby(lambda x: x.name, deps)
+
+                bad_deps.extend([[spec, MatchSpec.union(_)[0]] for _ in deps.values()])
         bad_deps = self._classify_bad_deps(bad_deps, specs_to_add, history_specs,
                                            strict_channel_priority)
-        raise UnsatisfiableError(bad_deps, strict=strict_channel_priority)
+        return bad_deps
 
     def _get_strict_channel(self, package_name):
         channel_name = None
@@ -467,6 +449,17 @@ class Resolve(object):
             return False
         return ms.strictness < specs_by_name[0].strictness
 
+    def _get_package_pool(self, specs):
+        specs = frozenset(specs)
+        if specs in self._pool_cache:
+            pool = self._pool_cache[specs]
+        else:
+            pool = self.get_reduced_index(specs)
+            grouped_pool = groupby(lambda x: x.name, pool)
+            pool = {k: set(v) for k, v in iteritems(grouped_pool)}
+            self._pool_cache[specs] = pool
+        return pool
+
     @time_recorder(module_name=__name__)
     def get_reduced_index(self, explicit_specs, sort_by_exactness=True):
         # TODO: fix this import; this is bad
@@ -492,7 +485,7 @@ class Resolve(object):
             # prioritize specs that are more exact.  Exact specs will evaluate to 3,
             #    constrained specs will evaluate to 2, and name only will be 1
             explicit_specs = sorted(list(explicit_specs), key=lambda x: (
-                exactness_and_number_of_deps(self, x), x.name), reverse=True)
+                exactness_and_number_of_deps(self, x), x.dist_str()), reverse=True)
         # tuple because it needs to be hashable
         explicit_specs = tuple(explicit_specs)
 
@@ -990,27 +983,37 @@ class Resolve(object):
     def get_conflicting_specs(self, specs):
         if not specs:
             return ()
-        reduced_index = self.get_reduced_index(tuple(specs), sort_by_exactness=False)
+        reduced_index = self.get_reduced_index(specs)
+
+        # Check if satisfiable
+        def mysat(specs, add_if=False):
+            constraints = r2.generate_spec_constraints(C, specs)
+            return C.sat(constraints, add_if)
+
         r2 = Resolve(reduced_index, True, channels=self.channels)
-        scp = context.channel_priority == ChannelPriority.STRICT
-        unsat_specs = {s for s in specs if not r2.find_matches_with_strict(s, scp)}
-        if not unsat_specs:
-            return ()
+        C = r2.gen_clauses()
+        solution = mysat(specs, True)
+        if solution:
+            final_unsat_specs = ()
         else:
             # This first result is just a single unsatisfiable core. There may be several.
+            unsat_specs = list(minimal_unsatisfiable_subset(specs, sat=mysat))
             satisfiable_specs = set(specs) - set(unsat_specs)
 
             # In this loop, we test each unsatisfiable spec individually against the satisfiable
             # specs to ensure there are no other unsatisfiable specs in the set.
             final_unsat_specs = set()
             while unsat_specs:
-                this_spec = unsat_specs.pop()
+                this_spec = unsat_specs.pop(0)
                 final_unsat_specs.add(this_spec)
-                test_specs = tuple(concatv((this_spec, ), satisfiable_specs))
-                r2 = Resolve(self.get_reduced_index(test_specs), True, channels=self.channels)
-                _unsat_specs = {s for s in specs if not r2.find_matches_with_strict(s, scp)}
-                unsat_specs.update(_unsat_specs - final_unsat_specs)
-                satisfiable_specs -= set(unsat_specs)
+                test_specs = satisfiable_specs | {this_spec}
+                C = r2.gen_clauses()  # TODO: wasteful call, but Clauses() needs refactored
+                solution = mysat(test_specs, True)
+                if not solution:
+                    these_unsat = minimal_unsatisfiable_subset(test_specs, sat=mysat)
+                    if len(these_unsat) > 1:
+                        unsat_specs.extend(these_unsat)
+                        satisfiable_specs -= set(unsat_specs)
         return tuple(final_unsat_specs)
 
     def bad_installed(self, installed, new_specs):
@@ -1094,7 +1097,9 @@ class Resolve(object):
 
     def install(self, specs, installed=None, update_deps=True, returnall=False):
         specs, preserve = self.install_specs(specs, installed or [], update_deps)
-        pkgs = self.solve(specs, returnall=returnall, _remove=False)
+        pkgs = []
+        if specs:
+            pkgs = self.solve(specs, returnall=returnall, _remove=False)
         self.restore_bad(pkgs, preserve)
         return pkgs
 
@@ -1137,24 +1142,45 @@ class Resolve(object):
         return pkgs
 
     @time_recorder(module_name=__name__)
-    def solve(self, specs, returnall=False, _remove=False, specs_to_add=None, history_specs=None):
+    def solve(self, specs, returnall=False, _remove=False, specs_to_add=None, history_specs=None,
+              repodata_fn=REPODATA_FN):
         # type: (List[str], bool) -> List[PackageRecord]
         if log.isEnabledFor(DEBUG):
-            log.debug('Solving for: %s', dashlist(sorted(text_type(s) for s in specs)))
+            dlist = dashlist(text_type(
+                '%i: %s target=%s optional=%s' % (i, s, s.target, s.optional))
+                for i, s in enumerate(specs))
+            log.debug('Solving for: %s', dlist)
 
         if specs and not isinstance(specs[0], MatchSpec):
             specs = tuple(MatchSpec(_) for _ in specs)
         specs = set(specs)
 
+        if not specs:
+            return tuple()
+
         # Find the compliant packages
         log.debug("Solve: Getting reduced index of compliant packages")
         len0 = len(specs)
 
         reduced_index = self.get_reduced_index(specs)
         if not reduced_index:
-            if len(specs) == 1 or any(self.get_reduced_index((s,)) for s in specs):
-                self.find_conflicts(specs, specs_to_add, history_specs)
-            return False if reduced_index is None else ([[]] if returnall else [])
+            # something is intrinsically unsatisfiable - either not found or not the right version
+            not_found_packages = set()
+            wrong_version_packages = set()
+            for s in specs:
+                if not self.find_matches(s):
+                    if s.name in self.groups:
+                        wrong_version_packages.add(s)
+                    else:
+                        not_found_packages.add(s)
+            if not_found_packages:
+                raise ResolvePackageNotFound(not_found_packages)
+            elif wrong_version_packages:
+                raise UnsatisfiableError([[d] for d in wrong_version_packages], chains=False)
+            if repodata_fn != REPODATA_FN:
+                # bail out until we have the full repodata.
+                raise UnsatisfiableError({})
+            self.find_conflicts(specs, specs_to_add, history_specs)
 
         # Check if satisfiable
         log.debug("Solve: determining satisfiability")
@@ -1185,7 +1211,11 @@ class Resolve(object):
         r2 = Resolve(reduced_index, True, channels=self.channels)
         C = r2.gen_clauses()
         solution = mysat(specs, True)
+
         if not solution:
+            if repodata_fn != REPODATA_FN:
+                # bail out until we have the full repodata.
+                raise UnsatisfiableError({})
             self.find_conflicts(specs, specs_to_add, history_specs)
 
         speco = []  # optional packages
@@ -1203,6 +1233,12 @@ class Resolve(object):
                 speca.append(s)
         speca.extend(MatchSpec(s) for s in specm)
 
+        if log.isEnabledFor(DEBUG):
+            log.debug('Requested specs: %s', dashlist(sorted(text_type(s) for s in specr)))
+            log.debug('Optional specs: %s', dashlist(sorted(text_type(s) for s in speco)))
+            log.debug('All other specs: %s', dashlist(sorted(text_type(s) for s in speca)))
+            log.debug('missing specs: %s', dashlist(sorted(text_type(s) for s in specm)))
+
         # Removed packages: minimize count
         log.debug("Solve: minimize removed packages")
         if _remove:
